package cn.com.qmth.print.manage.controller;

import cn.com.qmth.print.manage.config.PmConstants;
import cn.com.qmth.print.manage.service.ExamStudentService;
import cn.com.qmth.print.manage.service.query.ExamStudentQuery;
import cn.com.qmth.print.manage.utils.PathUtil;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.StatusException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.Objects;

/**
 * @Date: 2021/11/16.
 */
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/student")
@Aac(strict = false, auth = true)
public class ExamStudentController extends BaseController {

    @Resource
    private ExamStudentService examStudentService;

    /**
     * 导入考生数据
     *
     * @return
     */
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    @Transactional(rollbackFor = Exception.class)
    public Object importFile(@RequestParam Long examId, @RequestParam MultipartFile file) throws IOException {
        String fileName = file.getOriginalFilename();
        if (fileName == null || fileName.length() == 0) {
            throw new StatusException("文件名不存在");
        }
        if (fileName.endsWith(PmConstants.XLSX)) {
            return examStudentService.importStudents(examId, file);
        } else if (fileName.endsWith(PmConstants.ZIP)) {
            return examStudentService.analyzeZipAndImportStudents(examId, file);
        } else {
            throw new StatusException(String.format("只允许导入后缀为[%s,%s]的文件", PmConstants.XLSX, PmConstants.ZIP));
        }
    }

    /**
     * 导出云阅卷考生数据
     *
     * @param response response
     * @param examId   考试id
     * @throws IOException 异常
     */
    @RequestMapping(value = "/export", method = RequestMethod.POST)
    public void exportExamStudent(HttpServletResponse response, @RequestParam("examId") Long examId) throws IOException {
        examStudentService.exportExamStudent(response, examId);
    }

    /**
     * 分页考生详情
     *
     * @return
     */
    @RequestMapping(value = "/detail", method = RequestMethod.POST)
    public Object studentDetail(@Validated ExamStudentQuery query) {
        return examStudentService.pageStudent(query);
    }

    @RequestMapping(value = "/template", method = RequestMethod.POST)
    public void getImportTemplate() {
        String resourcePath = PathUtil.getResoucePath("importtemplates/studentImportTemplate.xlsx");
        if (Objects.isNull(resourcePath)) {
            throw new StatusException("未找到模板路径");
        }
        exportFile("考生导入模板.xlsx", new File(resourcePath));
    }
}