package cn.com.qmth.print.manage.service.impl;

import cn.com.qmth.print.manage.config.PmConstants;
import cn.com.qmth.print.manage.config.SysProperty;
import cn.com.qmth.print.manage.dao.ExamDao;
import cn.com.qmth.print.manage.dao.ExamStudentDao;
import cn.com.qmth.print.manage.dto.ExamStudentExportDTO;
import cn.com.qmth.print.manage.dto.ExamStudentImportDTO;
import cn.com.qmth.print.manage.dto.StudentDTO;
import cn.com.qmth.print.manage.entity.CheckRecordEntity;
import cn.com.qmth.print.manage.entity.ExamEntity;
import cn.com.qmth.print.manage.entity.ExamStudentEntity;
import cn.com.qmth.print.manage.enums.GroupType;
import cn.com.qmth.print.manage.enums.RecordStatus;
import cn.com.qmth.print.manage.service.CheckRecordService;
import cn.com.qmth.print.manage.service.ExamStudentService;
import cn.com.qmth.print.manage.service.query.ExamStudentQuery;
import cn.com.qmth.print.manage.utils.Zip4jUtil;
import cn.com.qmth.print.manage.utils.excel.ExcelError;
import cn.com.qmth.print.manage.utils.excel.ExcelReader;
import cn.com.qmth.print.manage.utils.excel.ExportExcel;
import cn.hutool.core.io.FileUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.crypto.AES;
import com.qmth.boot.tools.models.ByteArray;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.atomic.AtomicLong;
import java.util.stream.Collectors;

@Service
public class ExamStudentServiceImpl extends ServiceImpl<ExamStudentDao, ExamStudentEntity> implements
        ExamStudentService {

    @Resource
    private CheckRecordService checkRecordService;

    @Resource
    private ExamDao examDao;

    @Resource
    private SysProperty sysProperty;

    @Override
    public ExamStudentEntity findByExamIdAndExamNumber(Long examId, String examNumber) {
        if (examId == null) {
            throw new StatusException("examId不能为空");
        }
        if (StringUtils.isEmpty(examNumber)) {
            throw new StatusException("准考证号不能为空");
        }
        // 判断examNumber是否为小条码(去掉末尾和examNumber)
        String smallExamNumber = examNumber.substring(0, examNumber.length() - 1);
        ExamStudentEntity checkExamNumber = this.getOne(new QueryWrapper<ExamStudentEntity>()
                .lambda()
                .eq(ExamStudentEntity::getExamId, examId)
                .eq(ExamStudentEntity::getExamNumber, smallExamNumber));
        if (Objects.isNull(checkExamNumber)) {
            throw new StatusException("未找到考生[" + examNumber + "]");
        }
        return checkExamNumber;
    }

    @Override
    public Long findSortNoByExamIdAndGroupTypeAndSort(Long examId, GroupType type, String groupName, boolean isAse) {
        QueryWrapper<ExamStudentEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ExamStudentEntity> lw = wrapper.lambda();
        lw.eq(ExamStudentEntity::getExamId, examId);
        if (GroupType.COURSE.equals(type)) {
            lw.eq(ExamStudentEntity::getCourseCode, groupName);
        } else if (GroupType.EXAM_SITE.equals(type)) {
            lw.eq(ExamStudentEntity::getExamSite, groupName);
        } else if (GroupType.EXAM_ROOM.equals(type)) {
            lw.eq(ExamStudentEntity::getExamRoom, groupName);
        }
        if (isAse) {
            wrapper.orderByAsc("sort_no");
        } else {
            wrapper.orderByDesc("sort_no");
        }
        wrapper.last("LIMIT 1");
        ExamStudentEntity entity = this.getOne(wrapper);
        return entity.getSortNo();
    }

    @Override
    public List<ExamStudentEntity> listByExamId(Long examId) {
        QueryWrapper<ExamStudentEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudentEntity::getExamId, examId);
        return this.list(queryWrapper);
    }

    @Override
    public IPage<ExamStudentEntity> pageStudent(ExamStudentQuery query) {
        QueryWrapper<ExamStudentEntity> queryWrapper = new QueryWrapper<>();
        if (query.getExamId() == null) {
            throw new StatusException("请选择批次");
        }
        queryWrapper.lambda().eq(ExamStudentEntity::getExamId, query.getExamId());
        if (StringUtils.isNotBlank(query.getExamNumber())) {
            queryWrapper.lambda().like(ExamStudentEntity::getExamNumber, "%" + query.getExamNumber() + "%");
        }
        if (StringUtils.isNotBlank(query.getCourseCode())) {
            queryWrapper.lambda().like(ExamStudentEntity::getCourseCode, "%" + query.getCourseCode() + "%");
        }
        if (StringUtils.isNotBlank(query.getExamSite())) {
            queryWrapper.lambda().like(ExamStudentEntity::getExamSite, "%" + query.getExamSite() + "%");
        }
        if (StringUtils.isNotBlank(query.getExamRoom())) {
            queryWrapper.lambda().like(ExamStudentEntity::getExamRoom, "%" + query.getExamRoom() + "%");
        }
        Page page = new Page(query.getPageNumber(), query.getPageSize());
        return this.page(page, queryWrapper);
    }

    @Override
    public List<ExcelError> importStudents(Long examId, MultipartFile file) throws IOException {
        ExamEntity examEntity = examDao.selectById(examId);
        List<ExamStudentEntity> studentList = new ArrayList<>();
        ExcelReader excelReader = new ExcelReader(StudentDTO.class);
        List<ExcelError> excelErrors = excelReader.reader(file.getInputStream(), obj -> {
            try {
                StudentDTO dto = (StudentDTO) obj;
                String nullError = "";

                String examNumber = dto.getExamNumber();
                String studentCode = dto.getStudentCode();
                String name = dto.getName();
                String subjectCode = dto.getCourseCode();
                String subjectName = dto.getCourseName();
                String examSite = dto.getExamSite();

                if (examNumber == null || examNumber.length() == 0) {
                    nullError = nullError + "[准考证号]";
                }
                if (studentCode == null || studentCode.length() == 0) {
                    nullError = nullError + "[学号]";
                }
                if (name == null || name.length() == 0) {
                    nullError = nullError + "[姓名]";
                }
                if (subjectCode == null || subjectCode.length() == 0) {
                    nullError = nullError + "[科目代码]";
                }
                if (subjectName == null || subjectName.length() == 0) {
                    nullError = nullError + "[科目名称]";
                }
                if (nullError.length() > 0) {
                    throw new StatusException(nullError + "不能为空");
                }

                ExamStudentEntity examStudentEntity = new ExamStudentEntity();
                examStudentEntity.setExamId(examId);
                examStudentEntity.setOrgId(examEntity.getOrgId());
                examStudentEntity.setExamNumber(examNumber);
                examStudentEntity.setStudentCode(studentCode);
                examStudentEntity.setName(name);
                examStudentEntity.setCourseCode(subjectCode.concat(PmConstants.LING_SIGN).concat(subjectName));
                examStudentEntity.setExamSite(examSite);
                examStudentEntity.setExamRoom(dto.getExamRoom());
                examStudentEntity.setExamUnit(dto.getExamUnit());
                examStudentEntity.setCreateTime(new Date());
                examStudentEntity.setUpdateTime(new Date());
                studentList.add(examStudentEntity);
                return null;
            } catch (RuntimeException e) {
                // 手动回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                ExcelError excelError = new ExcelError();
                excelError.setExcelErrorType(e.getMessage());
                return excelError;
            }
        });
        String errors = errorsString(excelErrors);
        if (errors.length() > 0) {
            throw new StatusException(errors);
        }

        if (!CollectionUtils.isEmpty(studentList)) {
            insertData(examEntity, studentList);
        }
        return excelErrors;
    }

    @Transactional
    @Override
    public Object analyzeZipAndImportStudents(Long examId, MultipartFile file) {
        String tempPath = sysProperty.getTempDir() + "/importExamStudent";
        try {
            byte[] bytes = file.getBytes();
            byte[] keyArr = Arrays.copyOfRange(bytes, 0, 16);
            byte[] ivArr = Arrays.copyOfRange(bytes, 16, 32);
            byte[] contentArr = Arrays.copyOfRange(bytes, 32, bytes.length);
            String key = new String(keyArr, StandardCharsets.UTF_8);
            String iv = new String(ivArr, StandardCharsets.UTF_8);
            ByteArray byteArray = AES.decrypt(contentArr, key, iv);

            File decrypt = new File(tempPath + "/decrypt.zip");
            if (!decrypt.getParentFile().exists()) {
                decrypt.getParentFile().mkdirs();
            }
            byteArray.toFile(decrypt);
            String dirPath = tempPath + "/unzip";
            Zip4jUtil.unzip(decrypt.getPath(), dirPath);

            File zipDir = new File(dirPath);
            File[] firstDirArr = zipDir.listFiles();
            List<File> studentFileList = Arrays.stream(Objects.requireNonNull(firstDirArr))
                    .filter(e -> e.getName().endsWith("xlsx"))
                    .collect(Collectors.toList());
            if (CollectionUtils.isEmpty(studentFileList)) {
                throw new StatusException("文件解析失败：未找到导入的excel文件");
            }
            if (studentFileList.size() != 1) {
                throw new StatusException("文件解析失败：存在多个excel文件");
            }
            File studentExcel = studentFileList.get(0);

            ExamEntity examEntity = examDao.selectById(examId);
            List<ExamStudentEntity> studentList = new ArrayList<>();
            ExcelReader excelReader = new ExcelReader(ExamStudentImportDTO.class);
            List<ExcelError> excelErrors = excelReader.reader(FileUtil.getInputStream(studentExcel), obj -> {
                try {
                    String nullError = "";
                    ExamStudentImportDTO dto = (ExamStudentImportDTO) obj;
                    String examNumber = dto.getExamNumber();
                    String studentCode = dto.getStudentCode();
                    String name = dto.getStudentName();
                    String subjectCode = dto.getCourseCode();
                    String subjectName = dto.getCourseName();

                    if (examNumber == null || examNumber.length() == 0) {
                        nullError = nullError + "[准考证号]";
                    }
                    if (studentCode == null || studentCode.length() == 0) {
                        nullError = nullError + "[学号]";
                    }
                    if (name == null || name.length() == 0) {
                        nullError = nullError + "[姓名]";
                    }
                    if (subjectCode == null || subjectCode.length() == 0) {
                        nullError = nullError + "[科目代码]";
                    }
                    if (subjectName == null || subjectName.length() == 0) {
                        nullError = nullError + "[科目名称]";
                    }
                    if (nullError.length() > 0) {
                        throw new StatusException(nullError + "不能为空");
                    }

                    ExamStudentEntity examStudentEntity = new ExamStudentEntity();
                    examStudentEntity.setExamId(examId);
                    examStudentEntity.setOrgId(examEntity.getOrgId());
                    examStudentEntity.setExamNumber(examNumber);
                    examStudentEntity.setStudentCode(studentCode);
                    examStudentEntity.setName(name);
                    examStudentEntity.setCourseCode(subjectCode.concat(PmConstants.LING_SIGN).concat(subjectName));
                    examStudentEntity.setExamSite(dto.getExamPlaceCode());
                    examStudentEntity.setExamUnit(dto.getExamUnit());
                    examStudentEntity.setCreateTime(new Date());
                    examStudentEntity.setUpdateTime(new Date());
                    studentList.add(examStudentEntity);
                    return null;
                } catch (RuntimeException e) {
                    // 手动回滚
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    ExcelError excelError = new ExcelError();
                    excelError.setExcelErrorType(e.getMessage());
                    return excelError;
                }
            });

            String errors = errorsString(excelErrors);
            if (errors.length() > 0) {
                throw new StatusException(errors);
            }

            if (!CollectionUtils.isEmpty(studentList)) {
                insertData(examEntity, studentList);
            }
            return excelErrors;
        } catch (Exception e) {
            e.printStackTrace();
            throw new StatusException(e.getMessage());
        } finally {
            FileUtil.del(tempPath);
        }
    }

    @Override
    public void exportExamStudent(HttpServletResponse response, Long examId) throws IOException {
        List<ExamStudentExportDTO> examStudentExportDTOList = this.baseMapper.findExportData(examId);
        new ExportExcel("考生数据", ExamStudentExportDTO.class).setDataList(examStudentExportDTOList).write(response, "考生数据.xlsx")
                .dispose();
    }

    @Override
    public void deleteByExamId(Long examId) {
        UpdateWrapper<ExamStudentEntity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(ExamStudentEntity::getExamId, examId);
        this.remove(updateWrapper);
    }

    @Transactional
    public void insertData(ExamEntity examEntity, List<ExamStudentEntity> studentList) {
        // 考试批次的分组字段
        GroupType groupType = examEntity.getGroupType();

        List<ExamStudentEntity> dbStudentList = this.list(new QueryWrapper<ExamStudentEntity>()
                .lambda()
                .eq(ExamStudentEntity::getExamId, examEntity.getId()));

        //
        List<String> checkExamNumber = dbStudentList
                .stream()
                .map(ExamStudentEntity::getExamNumber)
                .distinct()
                .collect(Collectors.toList());
        List<String> examNumberList = studentList.stream().map(ExamStudentEntity::getExamNumber).collect(Collectors.toList());
        for (String examNumber : examNumberList) {
            if (checkExamNumber.contains(examNumber)) {
                throw new StatusException(String.format("导入失败,已存在准考证号为[%s]的考生", examNumber));
            } else {
                checkExamNumber.add(examNumber);
            }
        }

        Map<String, List<ExamStudentEntity>> groupCollection = null;
        switch (groupType) {
            case COURSE:
                if (studentList.stream().anyMatch(e -> e.getCourseCode() == null || e.getCourseCode().length() == 0)){
                    throw new StatusException(String.format("不能导入分组字段[%s]为空的考生文件",groupType.getName()));
                }
                // 导入的新考生数据不能包含有之前科目的
                List<String> dbCourseList = dbStudentList
                        .stream()
                        .filter(e -> e.getCourseCode() != null && e.getCourseCode().length() > 0)
                        .map(ExamStudentEntity::getCourseCode)
                        .distinct()
                        .collect(Collectors.toList());
                if (studentList.stream().anyMatch(e -> dbCourseList.contains(e.getCourseCode()))) {
                    throw new StatusException(String.format("分组字段[%s]重复导入考生", GroupType.COURSE.getName()));
                }
                groupCollection = studentList.stream().collect(Collectors.groupingBy(ExamStudentEntity::getCourseCode));
                break;
            case EXAM_SITE:
                if (studentList.stream().anyMatch(e -> e.getExamSite() == null || e.getExamSite().length() == 0)){
                    throw new StatusException(String.format("不能导入分组字段[%s]为空的考生文件",groupType.getName()));
                }
                // 导入的新考生数据不能包含有之前考点
                List<String> dbExamSiteList = dbStudentList
                        .stream()
                        .filter(e -> e.getExamSite() != null && e.getExamSite().length() > 0)
                        .map(ExamStudentEntity::getExamSite)
                        .distinct()
                        .collect(Collectors.toList());
                if (studentList.stream().anyMatch(e -> dbExamSiteList.contains(e.getExamSite()))) {
                    throw new StatusException(String.format("分组字段[%s]重复导入考生", GroupType.EXAM_SITE.getName()));
                }
                groupCollection = studentList.stream().collect(Collectors.groupingBy(ExamStudentEntity::getExamSite));
                break;
            case EXAM_ROOM:
                if (studentList.stream().anyMatch(e -> e.getExamRoom() == null || e.getExamRoom().length() == 0)){
                    throw new StatusException(String.format("不能导入分组字段[%s]为空的考生文件",groupType.getName()));
                }
                // 导入的新考生数据不能包含有之前考生
                List<String> dbExamRoomList = dbStudentList
                        .stream()
                        .filter(e -> e.getExamRoom() != null && e.getExamRoom().length() > 0)
                        .map(ExamStudentEntity::getExamRoom)
                        .distinct()
                        .collect(Collectors.toList());
                if (studentList.stream().anyMatch(e -> dbExamRoomList.contains(e.getExamRoom()))) {
                    throw new StatusException(String.format("分组字段[%s]重复导入考生", GroupType.EXAM_ROOM.getName()));
                }
                groupCollection = studentList.stream().collect(Collectors.groupingBy(ExamStudentEntity::getExamRoom));
                break;
        }

        // 添加考生
        Long sort = dbStudentList.stream().map(ExamStudentEntity::getSortNo).max(Long::compare).orElse(0L);
        AtomicLong aLong = new AtomicLong(sort);
        studentList = studentList.stream().peek(e -> e.setSortNo(aLong.incrementAndGet())).collect(Collectors.toList());

        List<ExamStudentEntity> data = new ArrayList<>();
        for (ExamStudentEntity stu : studentList) {
            if (data.size() == 2000) {
                this.saveBatch(data);
                data.clear();
            }
            data.add(stu);
        }
        // 将剩下的数据也导入
        if (!data.isEmpty()) {
            this.saveBatch(data);
        }

        // 添加校验数据
        List<CheckRecordEntity> checkRecordEntities = new ArrayList<>();
        for (Map.Entry<String, List<ExamStudentEntity>> entry : groupCollection.entrySet()) {
            CheckRecordEntity checkRecordEntity = new CheckRecordEntity();
            checkRecordEntity.setOrgId(examEntity.getOrgId());
            checkRecordEntity.setExamId(examEntity.getId());
            checkRecordEntity.setGroupType(examEntity.getGroupType());
            checkRecordEntity.setGroupName(entry.getKey());
            checkRecordEntity.setStatus(RecordStatus.NONE);
            checkRecordEntity.setCreateTime(new Date());
            checkRecordEntity.setUpdateTime(new Date());
            checkRecordEntities.add(checkRecordEntity);
        }
        checkRecordService.saveBatch(checkRecordEntities);
    }

    /**
     * 拼接导入异常信息
     *
     * @param excelErrors 错误信息
     * @return 结果
     */
    private String errorsString(List<ExcelError> excelErrors) {
        StringJoiner sj = new StringJoiner(";");
        if (!excelErrors.isEmpty()) {
            int forint = excelErrors.size() < 10 ? excelErrors.size() : 9;
            for (int i = 0; i < forint; i++) {
                ExcelError excelError = excelErrors.get(i);
                sj.add("第" + excelError.getRow() + "行," + excelError.getExcelErrorType());
            }
        }
        return sj.toString();
    }
}