package cn.com.qmth.print.manage.service.impl;

import java.util.Date;
import java.util.List;

import javax.annotation.Resource;

import com.qmth.boot.core.exception.StatusException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import cn.com.qmth.print.manage.dao.OrgDao;
import cn.com.qmth.print.manage.entity.OrgEntity;
import cn.com.qmth.print.manage.service.OrgService;
import cn.com.qmth.print.manage.service.query.OrgQuery;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.solar.model.OrgInfo;
import com.qmth.boot.core.solar.service.SolarService;

@Service
public class OrgServiceImpl extends ServiceImpl<OrgDao, OrgEntity> implements OrgService {

    @Resource
    private SolarService solarService;

    @Override
    public IPage<OrgEntity> pageQuery(OrgQuery query) {
        QueryWrapper<OrgEntity> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotEmpty(query.getCode())) {
            queryWrapper.lambda().like(OrgEntity::getCode, query.getCode() + "%");
        }
        if (StringUtils.isNotEmpty(query.getName())) {
            queryWrapper.lambda().like(OrgEntity::getName, query.getName() + "%");
        }
        queryWrapper.lambda().orderByAsc(OrgEntity::getId);
        Page<OrgEntity> page = new Page<>(query.getPageNumber(), query.getPageSize());
        return this.page(page, queryWrapper);
    }

    @Override
    public List<OrgEntity> listByEnable() {
        QueryWrapper<OrgEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OrgEntity::getEnable, true).orderByAsc(OrgEntity::getId);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public void pull() {
        List<OrgInfo> orglist = solarService.getOrgList();
        List<OrgEntity> orgEntities = this.list();
        // 不在中心的学校，本地全部禁用
        if (!CollectionUtils.isEmpty(orgEntities)) {
            for (OrgEntity orgEntity : orgEntities) {
                orgEntity.setEnable(false);
                this.updateById(orgEntity);
            }
        }
        if (!orglist.isEmpty()) {
            for (OrgInfo org : orglist) {
                OrgEntity entity = null;
                if (!CollectionUtils.isEmpty(orgEntities)) {
                    entity = orgEntities.stream().filter(m -> m.getCode().equals(org.getCode())).findFirst()
                            .orElse(null);
                }
                if (entity == null) {
                    entity = new OrgEntity();
                    // 中心的学校ID被占用，则使用自增ID
                    OrgEntity orgById = this.getById(org.getId());
                    if (orgById == null) {
                        entity.setId(org.getId());
                    }
                }
                entity.setCode(org.getCode());
                entity.setName(org.getName());
                entity.setAccessKey(org.getAccessKey());
                entity.setAccessSecret(org.getAccessSecret());
                entity.setLogoUrl(org.getLogoUrl());
                entity.setEnable(true);
                entity.setCreateTime(new Date());
                entity.setUpdateTime(new Date());
                this.saveOrUpdate(entity);
            }
        }
    }

    @Override
    public OrgEntity findByCode(String code) {
        List<OrgEntity> orgEntityList = this.list(new QueryWrapper<OrgEntity>().lambda().eq(OrgEntity::getCode, code));
        if (CollectionUtils.isEmpty(orgEntityList)) {
            throw new StatusException(String.format("机构[%s]不存在", code));
        }
        if (orgEntityList.size() > 1) {
            throw new StatusException(String.format("存在多个编号为[%s]的机构", code));
        }
        return orgEntityList.get(0);
    }
}
