package cn.com.qmth.print.manage.service.impl;

import cn.com.qmth.print.manage.dao.UserDao;
import cn.com.qmth.print.manage.entity.UserEntity;
import cn.com.qmth.print.manage.enums.RoleMeta;
import cn.com.qmth.print.manage.service.ExamService;
import cn.com.qmth.print.manage.service.UserService;
import cn.com.qmth.print.manage.service.query.UserQuery;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.StatusException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @Date: 2021/11/16.
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserDao, UserEntity> implements UserService {
    @Resource
    private ExamService examService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean save(UserEntity requestUser, Long id, String loginName, String name, RoleMeta role, String password, boolean enable, MultipartFile file) throws IOException {
        UserEntity user = new UserEntity();
        Long requestUserId = requestUser.getId();
        if (id != null) {
            user.setId(id);
            UserEntity userEntity = this.getById(id);
            if (userEntity == null) {
                throw new StatusException("用户不存在");
            }
            if (!loginName.equals(userEntity.getLoginName())) {
                throw new StatusException("用户名不能修改");
            }
            user = userEntity;
        } else {
            // 新增时用户名不能重复
            if (this.count(new QueryWrapper<UserEntity>().lambda().eq(UserEntity::getLoginName,loginName)) > 0){
                throw new StatusException("该账号已存在");
            }
            user.setCreateTime(new Date());
        }

        RoleMeta requestUserRole = requestUser.getRole();
        switch (requestUserRole) {
            case ADMIN:
                if (!RoleMeta.PRINT_LEADER.equals(role)) {
                    throw new StatusException(String.format("[%s]只能创建[%s]角色的用户", RoleMeta.ADMIN.getName(), RoleMeta.PRINT_LEADER.getName()));
                }
                break;
            case PRINT_LEADER:
                if (!RoleMeta.CHECKER.equals(role)) {
                    throw new StatusException(String.format("[%s]只能创建[%s]角色的用户", RoleMeta.PRINT_LEADER.getName(), RoleMeta.CHECKER.getName()));
                }
                break;
            default:
                throw new StatusException(String.format("只有[%s][%s]角色的用户可以编辑用户", RoleMeta.ADMIN, RoleMeta.PRINT_LEADER));
        }

        user.setLoginName(loginName);
        user.setRole(role);
        if (StringUtils.isBlank(name)) {
            user.setName(loginName);
        } else {
            user.setName(name);
        }
        user.setCreatorId(requestUserId);
        user.setUpdaterId(requestUserId);
        if (password != null && password.length() > 0) {
            user.setPassword(password);
        }
        user.setEnable(enable);
        user.setUpdateTime(new Date());
        boolean result = this.saveOrUpdate(user);
        if (RoleMeta.ADMIN.equals(requestUser.getRole()) && Objects.nonNull(file)) {
            // 如果是管理员且包含导入文件-处理导入批次逻辑
            examService.importExams(requestUser, user.getId(), file);
        }
        return result;
    }

    @Override
    public IPage<UserEntity> pageQuery(Long printUserId, UserQuery query) {
        QueryWrapper<UserEntity> queryWrapper = new QueryWrapper<>();
        if (query.getUserId() != null && query.getUserId() > 0) {
            queryWrapper.lambda().eq(UserEntity::getId, query.getUserId());
        }
        if (query.getRole() != null) {
            queryWrapper.lambda().eq(UserEntity::getRole, query.getRole());
        }
        if (printUserId != null && printUserId > 0){
            // 如果是印点负责人，只能查看自己创建的校验员
            queryWrapper.lambda()
                    .eq(UserEntity::getCreatorId,printUserId)
                    .eq(UserEntity::getRole,RoleMeta.CHECKER);
        }
        queryWrapper.lambda().eq(UserEntity::isEnable, query.isEnable()).orderByAsc(UserEntity::getId);
        Page<UserEntity> page = new Page<>(query.getPageNumber(), query.getPageSize());
        return this.page(page, queryWrapper);
    }

    @Override
    public List<UserEntity> listByEnable() {
        QueryWrapper<UserEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(UserEntity::getRole, RoleMeta.CHECKER).eq(UserEntity::isEnable, true);
        return this.list(queryWrapper);
    }

    @Override
    public UserEntity findByLoginName(String loginName) {
        QueryWrapper<UserEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(UserEntity::getLoginName, loginName);
        return this.getOne(queryWrapper);
    }
}
