package cn.com.qmth.print.manage.utils;

import com.qmth.boot.core.exception.StatusException;
import net.lingala.zip4j.core.ZipFile;
import net.lingala.zip4j.exception.ZipException;

import java.io.File;
import java.util.Objects;

/**
 * @Description:
 * @Author: CaoZixuan
 * @Date:
 */
public class Zip4jUtil {

    /**
     * 解压zip
     *
     * @param scrPath  zip路径
     * @param destPath 解压后路径
     */
    public static void unzip(String scrPath, String destPath) {
        if (scrPath == null || scrPath.length() == 0) {
            throw new StatusException("原始路径不能为空");
        }
        if (destPath == null || destPath.length() == 0) {
            throw new StatusException("解压路径不能为空");
        }
        commonUnZipFile(scrPath, destPath, null);
    }

    /**
     * 解压zip带密码
     *
     * @param scrPath  zip路径
     * @param destPath 解压后路径
     * @param password 解压密码
     */
    public static void unzipEncryptFile(String scrPath, String destPath, String password) {
        if (scrPath == null || scrPath.length() == 0) {
            throw new StatusException("原始路径不能为空");
        }
        if (destPath == null || destPath.length() == 0) {
            throw new StatusException("解压路径不能为空");
        }
        if (password == null || password.length() == 0) {
            throw new StatusException("解压密码不能为空");
        }

        commonUnZipFile(scrPath, destPath, password);
    }

    /**
     * 解压zip包
     *
     * @param scrPath  zip路径
     * @param destPath 解压后路径
     * @param password 解压密码
     */
    private static void commonUnZipFile(String scrPath, String destPath, String password) {
        try {
            File file = new File(scrPath);
            if (!file.exists()) {
                file.getParentFile().mkdirs();
            }
            ZipFile zipFile = new ZipFile(scrPath);
            if (Objects.nonNull(password) && zipFile.isEncrypted()) {
                zipFile.setPassword(password);
            }
            File currentFile = new File(destPath);
            if (!currentFile.exists()) {
                currentFile.mkdirs();
            }
            zipFile.extractAll(destPath);
        } catch (ZipException e) {
            throw new StatusException(e.getMessage());
        }
    }
}
